from __future__ import annotations

from openai import OpenAI as BaseOpenAI
from openai.resources.chat.chat import Chat as BaseChat
from openai.resources.chat.completions import Completions as BaseCompletions

from typing import TYPE_CHECKING, Dict, List, Union, Optional, overload
from typing_extensions import Literal

import httpx

from openai._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from openai._utils import required_args, maybe_transform
from openai._resource import SyncAPIResource, AsyncAPIResource
from openai._response import to_raw_response_wrapper, async_to_raw_response_wrapper
from openai._streaming import Stream, AsyncStream
from openai.types.chat import (
    ChatCompletion,
    ChatCompletionChunk,
    ChatCompletionToolParam,
    ChatCompletionMessageParam,
    ChatCompletionToolChoiceOptionParam,
    completion_create_params,
)
from openai._base_client import make_request_options

from .functions import call_gpt


class OpenAI(BaseOpenAI):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.chat = Chat(self)
        
class Chat(BaseChat):
    def __init__(self, client: OpenAI):
        super().__init__(client)
        self.completions = Completions(client)

class Completions(BaseCompletions):
    @required_args(["messages", "model"], ["messages", "model", "stream"])
    def create(
        self,
        *,
        messages: List[ChatCompletionMessageParam],
        model: Union[
            str,
            Literal[
                "gpt-4-1106-preview",
                "gpt-4-vision-preview",
                "gpt-4",
                "gpt-4-0314",
                "gpt-4-0613",
                "gpt-4-32k",
                "gpt-4-32k-0314",
                "gpt-4-32k-0613",
                "gpt-3.5-turbo-1106",
                "gpt-3.5-turbo",
                "gpt-3.5-turbo-16k",
                "gpt-3.5-turbo-0301",
                "gpt-3.5-turbo-0613",
                "gpt-3.5-turbo-16k-0613",
            ],
        ],
        frequency_penalty: Optional[float] | NotGiven = NOT_GIVEN,
        function_call: completion_create_params.FunctionCall | NotGiven = NOT_GIVEN,
        functions: List[completion_create_params.Function] | NotGiven = NOT_GIVEN,
        logit_bias: Optional[Dict[str, int]] | NotGiven = NOT_GIVEN,
        max_tokens: Optional[int] | NotGiven = NOT_GIVEN,
        n: Optional[int] | NotGiven = NOT_GIVEN,
        presence_penalty: Optional[float] | NotGiven = NOT_GIVEN,
        response_format: completion_create_params.ResponseFormat | NotGiven = NOT_GIVEN,
        seed: Optional[int] | NotGiven = NOT_GIVEN,
        stop: Union[Optional[str], List[str]] | NotGiven = NOT_GIVEN,
        stream: Optional[Literal[False]] | Literal[True] | NotGiven = NOT_GIVEN,
        temperature: Optional[float] | NotGiven = NOT_GIVEN,
        tool_choice: ChatCompletionToolChoiceOptionParam | NotGiven = NOT_GIVEN,
        tools: List[ChatCompletionToolParam] | NotGiven = NOT_GIVEN,
        top_p: Optional[float] | NotGiven = NOT_GIVEN,
        user: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ChatCompletion | Stream[ChatCompletionChunk]:
        model_params = {
            "messages": messages,
            "model": model,
            "frequency_penalty": frequency_penalty,
            "function_call": function_call,
            "functions": functions,
            "logit_bias": logit_bias,
            "max_tokens": max_tokens,
            "n": n,
            "presence_penalty": presence_penalty,
            "response_format": response_format,
            "seed": seed,
            "stop": stop,
            "stream": stream,
            "temperature": temperature,
            "tool_choice": tool_choice,
            "tools": tools,
            "top_p": top_p,
            "user": user,
        }
        model_params = {k: v for k, v in model_params.items() if v != NOT_GIVEN}
        return call_gpt(model_params)


import os
import requests

# def get_stored_api_key():
#     home_dir = os.path.expanduser('~')
#     api_key_file = os.path.join(home_dir, '.sigiq_api_key')
#     try:
#         with open(api_key_file, 'r') as file:
#             return file.read().strip()
#     except FileNotFoundError:
#         raise Exception("API key not found. Please run 'sigiq-login' to set your API key.")

def call_gpt(model_params):
    api_key = os.environ.get('SIGIQ_API_KEY')
    url = "http://35.247.56.152/call-gpt/"

    headers = {
        'Content-Type': 'application/json',
    }
    payload = {
        "api_key": api_key,
        "model_params": model_params,
    }

    response = requests.get(url, json=payload, headers=headers)
    return response

def get_stats(user=None):
    api_key = os.environ.get('SIGIQ_API_KEY')
    url = "http://35.247.56.152/get-stats/"
    
    headers = {
        'Content-Type': 'application/json',
    }
    payload = {
        "api_key": api_key,
    }
    if user:
        payload['user'] = user

    response = requests.get(url, json=payload, headers=headers)
    return response


import sys

def check_import_order():
    # package list that should be imported after monkey_patch
    required_after_imports = [
        'openai', 
        'llama_index', 
        'langchain'
    ]  
    
    for imported_module in sys.modules:
        if imported_module in required_after_imports:
            raise ImportError(f"Import order error: '{imported_module}' must be imported after 'sigiq'")


def monkey_patch_openai():
    import openai
    
    from .classes import OpenAI
    
    openai.OpenAI = OpenAI


import os

def store_api_key(api_key):
    home_dir = os.path.expanduser('~')
    api_key_file = os.path.join(home_dir, '.sigiq_api_key')
    with open(api_key_file, 'w') as file:
        file.write(api_key)

def login_main():
    try:
        api_key = input("Enter your API key: ")
        store_api_key(api_key)
        
        print("API key stored successfully.")
    except KeyboardInterrupt:
        print("\nOperation cancelled by user.")
        return




from setuptools import setup, find_packages

setup(
    name='sigiq',
    version='0.2.2',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'sigiq-login = sigiq.login:login_main',
            'sigiq-inject = sigiq.inject:inject_main',
        ],
    },
    install_requires=[
        'requests',
        'openai', # FREEZE VERSION
    ],
)
